@@ -57,7 +57,6 @@ Street, Fifth Floor, Boston, MA 02110-1335 USA
 #include "backup_copy.h"
 #include "backup_mysql.h"
 #include <btr0btr.h>
-#include "xb0xb.h"
 
 #define ROCKSDB_BACKUP_DIR "#rocksdb"
 
@@ -203,9 +202,9 @@ datadir_iter_new(const char *path, bool skip_first_level = true)
 	pthread_mutex_init(&it->mutex, NULL);
 	it->datadir_path = strdup(path);
 
-	it->dir = os_file_opendir(it->datadir_path, TRUE);
+	it->dir = os_file_opendir(it->datadir_path);
 
-	if (it->dir == NULL) {
+	if (it->dir == IF_WIN(INVALID_HANDLE_VALUE, nullptr)) {
 
 		goto error;
 	}
@@ -236,11 +235,9 @@ bool
 datadir_iter_next_database(datadir_iter_t *it)
 {
 	if (it->dbdir != NULL) {
-		if (os_file_closedir(it->dbdir) != 0) {
-
+		if (os_file_closedir_failed(it->dbdir)) {
 			msg("Warning: could not"
 			      " close database directory %s", it->dbpath);
-
 			it->err = DB_ERROR;
 
 		}
@@ -286,10 +283,9 @@ datadir_iter_next_database(datadir_iter_t *it)
 
 		/* We want wrong directory permissions to be a fatal error for
 		XtraBackup. */
-		it->dbdir = os_file_opendir(it->dbpath, TRUE);
-
-		if (it->dbdir != NULL) {
+		it->dbdir = os_file_opendir(it->dbpath);
 
+		if (it->dir != IF_WIN(INVALID_HANDLE_VALUE, nullptr)) {
 			it->is_file = false;
 			return(true);
 		}
@@ -644,12 +640,10 @@ int
 mkdirp(const char *pathname, int Flags, myf MyFlags)
 {
 	char *parent, *p;
-	int len = strlen(pathname) + 1;
 
 	/* make a parent directory path */
-	if (!(parent= (char *)malloc(len)))
+	if (!(parent= strdup(pathname)))
           return(-1);
-	memcpy(parent, pathname, len);
 
 	for (p = parent + strlen(parent);
 	    !is_path_separator(*p) && p != parent; p--) ;
@@ -732,9 +726,9 @@ directory_exists(const char *dir, bool create)
 	}
 
 	/* could be symlink */
-	os_dir = os_file_opendir(dir, FALSE);
+	os_dir = os_file_opendir(dir);
 
-	if (os_dir == NULL) {
+	if (os_dir == IF_WIN(INVALID_HANDLE_VALUE, nullptr)) {
 		my_strerror(errbuf, sizeof(errbuf), my_errno);
 		msg("Can not open directory %s: %s", dir,
 			errbuf);
@@ -762,9 +756,9 @@ directory_exists_and_empty(const char *dir, const char *comment)
 		return(false);
 	}
 
-	os_dir = os_file_opendir(dir, FALSE);
+	os_dir = os_file_opendir(dir);
 
-	if (os_dir == NULL) {
+	if (os_dir == IF_WIN(INVALID_HANDLE_VALUE, nullptr)) {
 		msg("%s can not open directory %s", comment, dir);
 		return(false);
 	}
@@ -1018,7 +1012,6 @@ static int fix_win_file_permissions(const char *file)
 	ACL* pOldDACL;
 	SECURITY_DESCRIPTOR* pSD = NULL;
 	EXPLICIT_ACCESS ea = { 0 };
-	BOOL isWellKnownSID = FALSE;
 	PSID pSid = NULL;
 
 	GetSecurityInfo(hFile, SE_FILE_OBJECT, DACL_SECURITY_INFORMATION, NULL, NULL,
@@ -1038,7 +1031,7 @@ static int fix_win_file_permissions(const char *file)
 	ea.grfInheritance = CONTAINER_INHERIT_ACE | OBJECT_INHERIT_ACE;
 	ea.Trustee.TrusteeType = TRUSTEE_IS_UNKNOWN;
 	ACL* pNewDACL = 0;
-	DWORD err = SetEntriesInAcl(1, &ea, pOldDACL, &pNewDACL);
+	SetEntriesInAcl(1, &ea, pOldDACL, &pNewDACL);
 	if (pNewDACL)
 	{
 		SetSecurityInfo(hFile, SE_FILE_OBJECT, DACL_SECURITY_INFORMATION, NULL, NULL,
@@ -2185,7 +2178,9 @@ decrypt_decompress()
 */
 static bool backup_files_from_datadir(const char *dir_path)
 {
-	os_file_dir_t dir = os_file_opendir(dir_path, TRUE);
+	os_file_dir_t dir = os_file_opendir(dir_path);
+	if (dir == IF_WIN(INVALID_HANDLE_VALUE, nullptr)) return false;
+
 	os_file_stat_t info;
 	bool ret = true;
 	while (os_file_readdir_next_file(dir_path, dir, &info) == 0) {
