@@ -34,19 +34,7 @@
 #include "sql_handler.h"                        // mysql_ha_rm_tables
 #include "sp_cache.h"                     // sp_invalidate_cache
 #include <mysys_err.h>
-#ifdef WITH_WSREP
 #include "debug_sync.h"
-#endif /* WITH_WSREP */
-
-LEX_CSTRING *make_lex_string(LEX_CSTRING *lex_str,
-                             const char* str, size_t length,
-                             MEM_ROOT *mem_root)
-{
-  if (!(lex_str->str= strmake_root(mem_root, str, length)))
-    return 0;
-  lex_str->length= length;
-  return lex_str;
-}
 
 /*************************************************************************/
 
@@ -304,7 +292,7 @@ class Deprecated_trigger_syntax_handler : public Internal_error_handler
 private:
 
   char m_message[MYSQL_ERRMSG_SIZE];
-  LEX_CSTRING *m_trigger_name;
+  const LEX_CSTRING *m_trigger_name;
 
 public:
 
@@ -320,8 +308,23 @@ public:
     if (sql_errno != EE_OUTOFMEMORY &&
         sql_errno != ER_OUT_OF_RESOURCES)
     {
+      // Check if the current LEX contains a non-empty spname
       if(thd->lex->spname)
         m_trigger_name= &thd->lex->spname->m_name;
+      else if (thd->lex->sphead)
+      {
+        /*
+          Some SP statements, for example IF, create their own local LEX.
+          All LEX instances are available in the LEX stack in sphead::m_lex.
+          Let's find the one that contains a non-zero spname.
+          Note, although a parse error has happened, the LEX instances
+          in sphead::m_lex are not freed yet at this point. The first
+          found non-zero spname contains the valid trigger name.
+        */
+        const sp_name *spname= thd->lex->sphead->find_spname_recursive();
+        if (spname)
+          m_trigger_name= &spname->m_name;
+      }
       if (m_trigger_name)
         my_snprintf(m_message, sizeof(m_message),
                     ER_THD(thd, ER_ERROR_IN_TRIGGER_BODY),
@@ -334,7 +337,7 @@ public:
     return false;
   }
 
-  LEX_CSTRING *get_trigger_name() { return m_trigger_name; }
+  const LEX_CSTRING *get_trigger_name() { return m_trigger_name; }
   char *get_error_message() { return m_message; }
 };
 
@@ -520,8 +523,7 @@ bool mysql_create_or_drop_trigger(THD *thd, TABLE_LIST *tables, bool create)
   }
 
 #ifdef WITH_WSREP
-  if (thd->wsrep_exec_mode == LOCAL_STATE)
-    WSREP_TO_ISOLATION_BEGIN(WSREP_MYSQL_DB, NULL, tables);
+  WSREP_TO_ISOLATION_BEGIN(WSREP_MYSQL_DB, NULL, tables);
 #endif
 
   /* We should have only one table in table list. */
@@ -593,6 +595,7 @@ bool mysql_create_or_drop_trigger(THD *thd, TABLE_LIST *tables, bool create)
                                                        STRING_WITH_LEN(act)));
                   };);
 #endif /* WITH_WSREP */
+
   result= (create ?
            table->triggers->create_trigger(thd, tables, &stmt_query):
            table->triggers->drop_trigger(thd, tables, &stmt_query));
@@ -635,9 +638,10 @@ end:
     my_ok(thd);
 
   DBUG_RETURN(result);
-
-WSREP_ERROR_LABEL:
+#ifdef WITH_WSREP
+wsrep_error_label:
   DBUG_RETURN(true);
+#endif
 }
 
 
@@ -1501,7 +1505,7 @@ bool Table_triggers_list::check_n_load(THD *thd, const LEX_CSTRING *db,
 
         if (unlikely(parse_error))
         {
-          LEX_CSTRING *name;
+          const LEX_CSTRING *name;
 
           /*
             In case of errors, disable all triggers for the table, but keep
@@ -1515,8 +1519,8 @@ bool Table_triggers_list::check_n_load(THD *thd, const LEX_CSTRING *db,
 
           if (likely((name= error_handler.get_trigger_name())))
           {
-            if (unlikely(!(make_lex_string(&trigger->name, name->str,
-                                           name->length, &table->mem_root))))
+            trigger->name= safe_lexcstrdup_root(&table->mem_root, *name);
+            if (unlikely(!trigger->name.str))
               goto err_with_lex_cleanup;
           }
           trigger->definer= ((!trg_definer || !trg_definer->length) ?
@@ -2325,12 +2329,10 @@ void Table_triggers_list::mark_fields_used(trg_event_type event)
         if (trg_field->field_idx != (uint)-1)
         {
           DBUG_PRINT("info", ("marking field: %d", trg_field->field_idx));
-          bitmap_set_bit(trigger_table->read_set, trg_field->field_idx);
           if (trg_field->get_settable_routine_parameter())
             bitmap_set_bit(trigger_table->write_set, trg_field->field_idx);
-          if (trigger_table->field[trg_field->field_idx]->vcol_info)
-            trigger_table->mark_virtual_col(trigger_table->
-                                            field[trg_field->field_idx]);
+          trigger_table->mark_column_with_deps(
+                                  trigger_table->field[trg_field->field_idx]);
         }
       }
     }
